from datetime import datetime

import app.database as database
import app.stock_exchange as stock_exchange
from app.types import Order


class FailedToProcessOrderError(Exception):
    pass


def process_order(db_connection, order_request, place_order=stock_exchange.place_order):
    order = Order(
        id="-1",
        created_at=datetime.now(),
        side=order_request.side,
        type=order_request.type_,
        instrument=order_request.instrument,
        limit_price=order_request.limit_price,
        quantity=order_request.quantity,
    )

    order = database.insert_order(db_connection, order)
    try:
        place_order(order)
        database.mark_order_as(db_connection, order.id_, "placed")
    except stock_exchange.OrderPlacementError:
        database.mark_order_as(db_connection, order.id_, "failed")
        raise FailedToProcessOrderError(order)
    finally:
        db_connection.commit()

    return order
